package com.ruoyi.oa.service.impl;

import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.oa.mapper.CmcCarMapper;
import com.ruoyi.oa.domain.CmcCar;
import com.ruoyi.oa.service.ICmcCarService;

/**
 * cmc车辆信息Service业务层处理
 * 
 * @author cmc
 * @date 2024-02-23
 */
@Service
public class CmcCarServiceImpl implements ICmcCarService 
{
    @Autowired
    private CmcCarMapper cmcCarMapper;

    /**
     * 查询cmc车辆信息
     * 
     * @param carId cmc车辆信息主键
     * @return cmc车辆信息
     */
    @Override
    public CmcCar selectCmcCarByCarId(Integer carId)
    {
        return cmcCarMapper.selectCmcCarByCarId(carId);
    }

    /**
     * 查询cmc车辆信息列表
     * 
     * @param cmcCar cmc车辆信息
     * @return cmc车辆信息
     */
    @Override
    public List<CmcCar> selectCmcCarList(CmcCar cmcCar)
    {
        return cmcCarMapper.selectCmcCarList(cmcCar);
    }

    /**
     * 新增cmc车辆信息
     * 
     * @param cmcCar cmc车辆信息
     * @return 结果
     */
    @Override
    public int insertCmcCar(CmcCar cmcCar)
    {
        return cmcCarMapper.insertCmcCar(cmcCar);
    }

    /**
     * 修改cmc车辆信息
     * 
     * @param cmcCar cmc车辆信息
     * @return 结果
     */
    @Override
    public int updateCmcCar(CmcCar cmcCar)
    {
        return cmcCarMapper.updateCmcCar(cmcCar);
    }

    /**
     * 批量删除cmc车辆信息
     * 
     * @param carIds 需要删除的cmc车辆信息主键
     * @return 结果
     */
    @Override
    public int deleteCmcCarByCarIds(Long[] carIds)
    {
        return cmcCarMapper.deleteCmcCarByCarIds(carIds);
    }

    /**
     * 删除cmc车辆信息信息
     * 
     * @param carId cmc车辆信息主键
     * @return 结果
     */
    @Override
    public int deleteCmcCarByCarId(Long carId)
    {
        return cmcCarMapper.deleteCmcCarByCarId(carId);
    }
}
